library(rhlra)
source("klminimizer.R")

get_w_projector <- function(signal, r, W) {
    N <- length(signal)
    obj <- list()
    class(obj) <- c("1d", "compensated")
    glrr <- get_glrr_from_nonlrf_series.1d(obj, signal, r)
    tangent_space <- get_space_by_glrr(obj, N, glrr, glrr_2 = TRUE)
    tangent_space %*% solve(t(tangent_space) %*% W %*% tangent_space,
                   t(tangent_space) %*% W)
}

get_limit_projector <- function(signal, L, r, alpha, p_coefs = numeric(0)) {
    N <- length(signal)
    Lmat_diags <- inv_ac_diags(L, p_coefs)

    if (length(p_coefs) == 0) {
        Rdiag <- kloptimw(N, L, alpha)
    } else {
        Rdiag <- kl_full_optimw(inv_ac_diags(N, p_coefs), Lmat_diags, alpha)
    }

    cadzow_weights_diags <- lapply(Lmat_diags, function(i) ssa_convolve(i, Rdiag))
    cadzow_weights <- bandSparse(N, k = 0:length(p_coefs),
                                           diag = cadzow_weights_diags, symm = TRUE)
    get_w_projector(signal, r, cadzow_weights)
}

calh_basis <- function(N, L) {
    asvec <- as.numeric(trmat_indices(N, L))
    Z <- length(asvec)
    N <- max(asvec)
    answer <- matrix(0, nrow = Z, ncol = N)
    for (i in 1:N) {
        answer[, i] <- as.numeric(asvec == i)
    }
    Matrix(answer, sparse = TRUE)
}

caln_basis <- function(N, L, v) {
    K <- N - L + 1
    r <- length(v) - 1
    left_basis_list <- list()
    right_basis_list <- list()
    obj <- list()
    class(obj) <- c("1d", "compensated")

    U <- get_space_by_glrr(obj, L, v)
    V <- get_space_by_glrr(obj, K, v)

    for (i in 1:r) {
        # left numbers basis
        left_basis <- matrix(0, nrow = L * K, ncol = L)
        for (j in 1:L) {
            right_draft <- matrix(0, nrow = L, ncol = K)
            right_draft[j, ] <- V[, i]
            left_basis[, j] <- right_draft
        }
        left_basis_list[[i]] <- left_basis

        # right numbers basis
        right_basis <- matrix(0, nrow = L * K, ncol = K)
        for (j in 1:K) {
            left_draft <- matrix(0, nrow = L, ncol = K)
            left_draft[, j] <- U[, i]
            right_basis[, j] <- left_draft
        }
        right_basis_list[[i]] <- right_basis

    }

    full_deriv_basis <- matrix(0, nrow = L * K)
    for (i in 1:r) {
        full_deriv_basis <- cbind(full_deriv_basis, left_basis_list[[i]], right_basis_list[[i]])
    }

    # print(full_deriv_basis)

    svdobj <- svd(full_deriv_basis)

    # print(svdobj$d)
    svdobj$u[, 1:((L+K)*r - r^2)]
}

get_oneit_projector <- function(signal, L, r, alpha, p_coefs = numeric(0)) {
    N <- length(signal)
    K <- N - L + 1

    Lmat_diags <- inv_ac_diags(L, p_coefs)

    Lmat <- band_mat_from_diags(Lmat_diags)

    Rdiag <- NULL

    if (length(p_coefs) == 0) {
        Rdiag <- kloptimw(N, L, alpha)
    } else {
        Rdiag <- kl_full_optimw(inv_ac_diags(N, p_coefs), Lmat_diags, alpha)
    }

    Lmat_summary <- summary(Lmat)

    matrix_cov_i <- as.vector(outer(Lmat_summary$i, 1:K, function(i, j) i + (j - 1)*L))
    matrix_cov_j <- as.vector(outer(Lmat_summary$j, 1:K, function(i, j) i + (j - 1)*L))
    matrix_cov_el <- as.vector(outer(Lmat_summary$x, Rdiag))
    matrix_cov <- sparseMatrix(matrix_cov_i, matrix_cov_j, x = matrix_cov_el, symmetric = TRUE)

    obj <- list()
    v <- get_glrr_from_nonlrf_series.1d(obj, signal, r)

    h_b <- calh_basis(N, L)

    n_b <- caln_basis(N, L, v)

    #canonical correlations formula -- equivalent to bottom
    solve(t(h_b) %*% matrix_cov %*% h_b) %*% (t(h_b) %*% matrix_cov %*% n_b) %*% solve(t(n_b) %*% matrix_cov %*% n_b) %*% (t(n_b) %*% matrix_cov %*% h_b)

    # proj_h_to_series <- solve(t(h_b) %*% matrix_cov %*% h_b, t(h_b) %*% matrix_cov)
    # proj_n_from_series <- n_b %*% solve(t(n_b) %*% (as.matrix(matrix_cov %*% n_b)), t(n_b) %*% as.matrix(matrix_cov %*% h_b))

    # as.matrix(proj_h_to_series) %*% proj_n_from_series
}

eigen_example <- function(gap_plot = FALSE) {
    N <- 40
    L <- 20
    signal <- sin((1:N) * 2 * pi / 6)
    r <- 2
    alphas <- exp(seq(from = log(0.0001), to = log(1), length.out = 30))

    values <- sapply(alphas, function(alpha) {
        projmat <- get_oneit_projector(signal, L, r, alpha)
        Mod(eigen(projmat)$values[2*r + 1])
    })

    if (!gap_plot) {
        plot(alphas, values, log = "x", type = "b",
             xlab = expression(alpha), ylab = "(2r+1)-th eigenvalue",
             xaxt = "n")
    } else {
        plot(alphas, 1 - values, log = "xy", type = "b",
             xlab = expression(alpha), ylab = "1 - (2r+1)-th eigenvalue",
             xaxt = "n")
    }
    axis(1, at = exp(seq(from = log(0.0001), to = log(1), length.out = 5)),
         labels = c(expression(10^-4), expression(10^-3), expression(10^-2),
                    expression(10^-1), expression(1)))
}
