library(Matrix)
library(rhlra)

convolve_with_1 <- function(x, L) {
    K <- length(x)
    N <- K + L - 1
    answer <- numeric(N)
    answer[1:K] <- x
    answer[(N-K+2):(N)] <- answer[(N-K+2):(N)] - x[-K]
    cumsum(answer)
}

summate_with_1 <- function(x, L) {
    x <- c(0, x)
    Np <- length(x)
    K <- Np - L
    x <- cumsum(x)
    x[(Np - K + 1):Np] - x[1:K]
}

kloptimw <- function(N, L, alpha, sigmasq = rep(1, N)) {
    K <- N - L + 1

    counter_fun <- 0
    counter_grad <- 0


    optfun <- function(x, L, sigmas, alpha) {
        wgts <- convolve_with_1(x, L) * sigmasq

        C <- N/sum(wgts)
        wgts <- wgts * C

        counter_fun <<- counter_fun + 1

        sum(wgts - log(wgts))
    }

    gradfun <- function(x, L, sigmas, alpha) {
        wgts <- convolve_with_1(x, L)

        C <- N/sum(wgts * sigmasq)

        counter_grad <<- counter_fun + 1

        dC <- - N/(sum(wgts * sigmasq)^2)

        summate_with_1(dC * wgts * sigmasq + C * sigmasq - dC/C - 1/wgts, L)
    }

    soltry <- optim(rep((alpha + 1)/2, K), optfun, gradfun, method = "L-BFGS-B",
                    lower = rep(alpha, K), upper = rep(1, K),
                    L = L, sigmas = sigmas, alpha = alpha)

    soltry$par
}

# plot(convolve_with_1(kloptimw(1000, 500, .1), 500))
#
# plot(convolve_with_1(kloptimw(1000, 500, .1, 1:1000), 500))
#
# get_matrix_weight_matrix(inv_ac_diags(50, .9), list(rep(1, 100)))

kl_full_optimw <- function(W_0_diags, left_diags, alpha) {
    N <- length(W_0_diags[[1]])
    L <- length(left_diags[[1]])
    K <- N - L + 1

    left_diags_dgtm <- as(band_mat_from_diags(left_diags), "TsparseMatrix")

    left_diags_i <- left_diags_dgtm@i
    left_diags_j <- left_diags_dgtm@j
    left_diags_data <- left_diags_dgtm@x

    get_padded_traces <- function(W) {
        sapply(1:K, function(i) {
            sum(W[cbind(left_diags_i + i, left_diags_j + i)] * left_diags_data)
        })
    }

    W_0 <- band_mat_from_diags(W_0_diags)
    sigma_0 <- as.matrix(solve(W_0))

    linear_coefs <- get_padded_traces(sigma_0)

    optfun <- function(x) {
        - determinant(get_matrix_weight_matrix(left_diags, list(x)), log = TRUE)$modulus + N * log(sum(x * linear_coefs))
    }

    gradfun <- function(x) {
        W <- get_matrix_weight_matrix(left_diags, list(x))
        sigma <- as.matrix(solve(W))

        det_grad <- -get_padded_traces(sigma)

        log_grad <- N * linear_coefs/(sum(x * linear_coefs))
        det_grad + log_grad
    }


    soltry <- optim(rep((alpha + 1)/2, K), optfun, gradfun, method = "L-BFGS-B",
                    lower = rep(alpha, K), upper = rep(1, K))

    soltry$par
}

# plot(kl_full_optimw(inv_ac_diags(1000, .9), inv_ac_diags(500, .9), .1))
# plot(kloptimw(200, 50, .1))
