source("mixed_cadzows.R")
source("mixed_linear_cadzows.R")

one_sin_signal <- sin((1:40) * 2 * pi / 6)
three_sin_signal <- sin((1:40) * 2 * pi / 6) + 0.5 * sin((1:40) * 2 * pi / 8) - 0.3 * sin((1:40) * 2 * pi / 12)
alpha <- 0.02

# uncomment lines below to perform time-consuming simulations

# data_linear <- mixed_linear_cadzow_example_multiple_alpha()
#
# cl <- NULL
# cl <- make_cluster()
# data_real <- mixed_cadzow_example_multiple_alpha(cl, sample_size = 1000)
# data_convergence <- slow_fast_convergence_multiple_I(cl, signal = one_sin_signal, r = 2, alpha = alpha, sample_size = 1000)
# data_r_too_big <- slow_fast_convergence_multiple_I(cl = cl, signal = one_sin_signal, r = 4, alpha = alpha, sample_size = 1000)
# data_r_6 <- slow_fast_convergence_multiple_I(cl = cl, signal = three_sin_signal, r = 6, alpha = alpha, sample_size = 1000)
#
# stopCluster(cl)
#
# save(data_linear, data_real, data_convergence, data_r_too_big, data_r_6, file = "simul_data.RData")

load("simul_data.RData")

alphas <- c(0.5, 0.1, 0.01)
I_1 <- 0:15
double_alphas <- c(alphas, alphas)


pdf("alphas.pdf", width = 8, height = 6)
par(mar = c(5, 4, 2, 1))

matplot(I_1, cbind(data_linear, data_real), type = "b",
        pch=c(1:length(alphas), 1:length(alphas)),
        ylab = "RMSE", xlab = substitute(paste("Number of iterations ", italic("T"[1]))))
legend("topright",
       legend = c(sapply(alphas, function(x) paste(x, ", Linear", sep = "")),
                  sapply(alphas, function(x) paste(x, ", True", sep = ""))),
       pch=c(1:length(alphas), 1:length(alphas)), col = 1:(length(alphas) * 2))

dev.off()

pdf("eigen.pdf", width = 4, height = 4)
par(mar = c(5, 4, 2, 1))

eigen_example()

dev.off()

pdf("eigen_2.pdf", width = 4, height = 4)
par(mar = c(5, 4, 2, 1))

eigen_example(TRUE)

dev.off()

pdf("convergence.pdf", width = 8, height = 6)
par(mar = c(5, 4, 2, 1))

matplot(default_I_sum, data_convergence, type = "b", pch=1:length(default_I_1), lty = 1,
        xlab = "Iteration", ylab = "RMSE")
legend("topright",
       legend = paste(alpha, "x", default_I_1, "+", max(default_I_1)-default_I_1, "x", 1),
       pch=1:length(default_I_1),
       col = 1:length(default_I_1))

dev.off()

pdf("r4.pdf", width = 8, height = 6)
par(mar = c(5, 4, 2, 1))

matplot(default_I_sum, data_r_too_big, type = "b", pch=1:length(default_I_1), lty = 1,
        xlab = "Iteration", ylab = "RMSE")
legend("topright",
       legend = paste(alpha, "x", default_I_1, "+", max(default_I_1) - default_I_1, "x", 1),
       pch=1:length(default_I_1),
       col = 1:length(default_I_1))

dev.off()

pdf("r6.pdf", width = 8, height = 6)
par(mar = c(5, 4, 2, 1))

matplot(default_I_sum, data_r_6, type = "b", pch=1:length(default_I_1), lty = 1,
        xlab = "Iteration", ylab = "RMSE")
legend("topright",
       legend = paste(alpha, "x", default_I_1, "+", max(default_I_1) - default_I_1, "x", 1),
       pch=1:length(default_I_1),
       col = 1:length(default_I_1))

dev.off()
