source("eigennumber.R")
library("expm")

get_mixed_projector <- function(signal, L, r, alpha_1, alpha_2, I_1, I_2, p_coefs = numeric(0)) {
    N <- length(signal)

    M_1 <- NULL
    M_2 <- NULL

    if (I_1 == 0) {
        M_1 <- diag(N)
    } else {
        if (is.infinite(I_1)) {
            M_1 <- as.matrix(get_limit_projector(signal, L, r, alpha_1, p_coefs))
        } else {
            M_1 <- as.matrix(get_oneit_projector(signal, L, r, alpha_1, p_coefs)) %^% I_1
        }
    }

    if (I_2 == 0) {
        M_2 <- diag(N)
    } else {
        if (is.infinite(I_2)) {
            M_2 <- as.matrix(get_limit_projector(signal, L, r, alpha_2, p_coefs))
        } else {
            M_2 <- as.matrix(get_oneit_projector(signal, L, r, alpha_2, p_coefs)) %^% I_2
        }
    }

    M_2 %*% M_1
}

MSE_linear_projector <- function(M, p_coefs = numeric(0), sgm = 1) {
    N <- dim(M)[1]
    inv_diags <- inv_ac_diags(N, p_coefs)
    sigma <- solve(bandSparse(N, k = 0:length(p_coefs),
                        diag = inv_diags, symm = TRUE)) * sgm^2
    mean(diag(M %*% sigma %*% t(M)))
}

mixed_linear_cadzow_example <- function(I_1 = 0:10, alpha_1 = 0.1, sigma = 0.1) {
    N <- 40
    L <- 20
    signal <- sin((1:N) * 2 * pi / 6)
    r <- 2

    alpha_2 <- 1
    I_2 <- Inf

    results <- sapply(I_1, function(I) {
        mp <- get_mixed_projector(signal, L, r, alpha_1, alpha_2, I, I_2)
        MSE_linear_projector(mp, sgm = sigma)
    })

    plot(I_1, results, type = "b")

    results
}

mixed_linear_cadzow_example_multiple_alpha <- function() {
    I_1 <- 0:15
    alphas <- c(0.5, 0.1, 0.01)
    results <- sapply(alphas, function(alpha) sqrt(mixed_linear_cadzow_example(I_1, alpha)))
    matplot(I_1, results, type = "b", pch=1:length(alphas))
    legend("topright", legend = alphas, pch=1:length(alphas),
           col = 1:length(alphas))
    results
}

# mixed_linear_cadzow_example()
# mixed_linear_cadzow_example_multiple_alpha()

alphas_limit_example <- function(alphas = exp(seq(from = log(0.001), to = log(1), length.out = 30))) {
    N <- 40
    L <- 20
    signal <- sin((1:N) * 2 * pi / 6)
    r <- 2

    results <- sapply(alphas, function(alpha) {
        mp <- get_limit_projector(signal, L, r, alpha)
        MSE_linear_projector(mp)
    })

    eigen_results <- sapply(alphas, function(alpha) {
        projmat <- get_oneit_projector(signal, L, r, alpha)
        -1/log(Mod(eigen(projmat)$values[2*r + 1]))
    })

    plot(alphas, eigen_results, type = "p", log="xy", xlab = "alpha", ylab = "-1/log(lambda)")

    plot(alphas, results, type = "b", log="x", xlab = "alpha", ylab = "MSE")

    plot(results, eigen_results, type = "p", log="y", xlab = "MSE", ylab = "-1/log(lambda)")

    results
}

# alphas_limit_example()
